"""Manage X10 functions."""


def all_units_off():
    """Send the X10 All Units OFF command."""


def all_lights_on():
    """Send the X10 All Lights ON command."""


def all_lights_off():
    """Send the X10 All Lights OFF command."""
